# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ostorlab/agent/message/proto/v3/use/device/device.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='ostorlab/agent/message/proto/v3/use/device/device.proto',
  package='ostorlab.agent.message.proto.v3.use.device',
  syntax='proto2',
  serialized_options=None,
  serialized_pb=_b('\n7ostorlab/agent/message/proto/v3/use/device/device.proto\x12*ostorlab.agent.message.proto.v3.use.device\"3\n\rLoginPassword\x12\x10\n\x08username\x18\x01 \x01(\t\x12\x10\n\x08password\x18\x02 \x01(\t\"1\n\x06SSHKey\x12\x13\n\x0bprivate_key\x18\x01 \x01(\t\x12\x12\n\npublic_key\x18\x02 \x01(\t\"\xb8\x01\n\x0b\x43redentials\x12S\n\x0elogin_password\x18\x01 \x01(\x0b\x32\x39.ostorlab.agent.message.proto.v3.use.device.LoginPasswordH\x00\x12\x45\n\x07ssh_key\x18\x02 \x01(\x0b\x32\x32.ostorlab.agent.message.proto.v3.use.device.SSHKeyH\x00\x42\r\n\x0b\x63redentials\":\n\x0b\x44\x65viceRelay\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07\x61\x64\x64ress\x18\x02 \x01(\t\x12\x0c\n\x04port\x18\x03 \x01(\x11\"*\n\x08\x44uration\x12\r\n\x05nanos\x18\x01 \x01(\x05\x12\x0f\n\x07seconds\x18\x02 \x01(\x03\"/\n\x08\x41rgument\x12\x10\n\x08\x61rg_name\x18\x01 \x01(\t\x12\x11\n\targ_value\x18\x02 \x03(\t\"\xd1\x03\n\x07Message\x12\x11\n\tdevice_id\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65vice_type\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65vice_name\x18\x03 \x01(\t\x12S\n\x12\x64\x65vice_credentials\x18\x04 \x01(\x0b\x32\x37.ostorlab.agent.message.proto.v3.use.device.Credentials\x12M\n\x0c\x64\x65vice_relay\x18\x05 \x01(\x0b\x32\x37.ostorlab.agent.message.proto.v3.use.device.DeviceRelay\x12Y\n\x18\x64\x65vice_relay_credentials\x18\x06 \x01(\x0b\x32\x37.ostorlab.agent.message.proto.v3.use.device.Credentials\x12\x46\n\x08\x64uration\x18\x07 \x01(\x0b\x32\x34.ostorlab.agent.message.proto.v3.use.device.Duration\x12\x42\n\x04\x61rgs\x18\x08 \x03(\x0b\x32\x34.ostorlab.agent.message.proto.v3.use.device.Argument')
)




_LOGINPASSWORD = _descriptor.Descriptor(
  name='LoginPassword',
  full_name='ostorlab.agent.message.proto.v3.use.device.LoginPassword',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='username', full_name='ostorlab.agent.message.proto.v3.use.device.LoginPassword.username', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='password', full_name='ostorlab.agent.message.proto.v3.use.device.LoginPassword.password', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=103,
  serialized_end=154,
)


_SSHKEY = _descriptor.Descriptor(
  name='SSHKey',
  full_name='ostorlab.agent.message.proto.v3.use.device.SSHKey',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='private_key', full_name='ostorlab.agent.message.proto.v3.use.device.SSHKey.private_key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='public_key', full_name='ostorlab.agent.message.proto.v3.use.device.SSHKey.public_key', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=156,
  serialized_end=205,
)


_CREDENTIALS = _descriptor.Descriptor(
  name='Credentials',
  full_name='ostorlab.agent.message.proto.v3.use.device.Credentials',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='login_password', full_name='ostorlab.agent.message.proto.v3.use.device.Credentials.login_password', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ssh_key', full_name='ostorlab.agent.message.proto.v3.use.device.Credentials.ssh_key', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='credentials', full_name='ostorlab.agent.message.proto.v3.use.device.Credentials.credentials',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=208,
  serialized_end=392,
)


_DEVICERELAY = _descriptor.Descriptor(
  name='DeviceRelay',
  full_name='ostorlab.agent.message.proto.v3.use.device.DeviceRelay',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='ostorlab.agent.message.proto.v3.use.device.DeviceRelay.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='address', full_name='ostorlab.agent.message.proto.v3.use.device.DeviceRelay.address', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='port', full_name='ostorlab.agent.message.proto.v3.use.device.DeviceRelay.port', index=2,
      number=3, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=394,
  serialized_end=452,
)


_DURATION = _descriptor.Descriptor(
  name='Duration',
  full_name='ostorlab.agent.message.proto.v3.use.device.Duration',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='nanos', full_name='ostorlab.agent.message.proto.v3.use.device.Duration.nanos', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seconds', full_name='ostorlab.agent.message.proto.v3.use.device.Duration.seconds', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=454,
  serialized_end=496,
)


_ARGUMENT = _descriptor.Descriptor(
  name='Argument',
  full_name='ostorlab.agent.message.proto.v3.use.device.Argument',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='arg_name', full_name='ostorlab.agent.message.proto.v3.use.device.Argument.arg_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='arg_value', full_name='ostorlab.agent.message.proto.v3.use.device.Argument.arg_value', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=498,
  serialized_end=545,
)


_MESSAGE = _descriptor.Descriptor(
  name='Message',
  full_name='ostorlab.agent.message.proto.v3.use.device.Message',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='device_id', full_name='ostorlab.agent.message.proto.v3.use.device.Message.device_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='device_type', full_name='ostorlab.agent.message.proto.v3.use.device.Message.device_type', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='device_name', full_name='ostorlab.agent.message.proto.v3.use.device.Message.device_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='device_credentials', full_name='ostorlab.agent.message.proto.v3.use.device.Message.device_credentials', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='device_relay', full_name='ostorlab.agent.message.proto.v3.use.device.Message.device_relay', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='device_relay_credentials', full_name='ostorlab.agent.message.proto.v3.use.device.Message.device_relay_credentials', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='duration', full_name='ostorlab.agent.message.proto.v3.use.device.Message.duration', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='args', full_name='ostorlab.agent.message.proto.v3.use.device.Message.args', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=548,
  serialized_end=1013,
)

_CREDENTIALS.fields_by_name['login_password'].message_type = _LOGINPASSWORD
_CREDENTIALS.fields_by_name['ssh_key'].message_type = _SSHKEY
_CREDENTIALS.oneofs_by_name['credentials'].fields.append(
  _CREDENTIALS.fields_by_name['login_password'])
_CREDENTIALS.fields_by_name['login_password'].containing_oneof = _CREDENTIALS.oneofs_by_name['credentials']
_CREDENTIALS.oneofs_by_name['credentials'].fields.append(
  _CREDENTIALS.fields_by_name['ssh_key'])
_CREDENTIALS.fields_by_name['ssh_key'].containing_oneof = _CREDENTIALS.oneofs_by_name['credentials']
_MESSAGE.fields_by_name['device_credentials'].message_type = _CREDENTIALS
_MESSAGE.fields_by_name['device_relay'].message_type = _DEVICERELAY
_MESSAGE.fields_by_name['device_relay_credentials'].message_type = _CREDENTIALS
_MESSAGE.fields_by_name['duration'].message_type = _DURATION
_MESSAGE.fields_by_name['args'].message_type = _ARGUMENT
DESCRIPTOR.message_types_by_name['LoginPassword'] = _LOGINPASSWORD
DESCRIPTOR.message_types_by_name['SSHKey'] = _SSHKEY
DESCRIPTOR.message_types_by_name['Credentials'] = _CREDENTIALS
DESCRIPTOR.message_types_by_name['DeviceRelay'] = _DEVICERELAY
DESCRIPTOR.message_types_by_name['Duration'] = _DURATION
DESCRIPTOR.message_types_by_name['Argument'] = _ARGUMENT
DESCRIPTOR.message_types_by_name['Message'] = _MESSAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

LoginPassword = _reflection.GeneratedProtocolMessageType('LoginPassword', (_message.Message,), dict(
  DESCRIPTOR = _LOGINPASSWORD,
  __module__ = 'ostorlab.agent.message.proto.v3.use.device.device_pb2'
  # @@protoc_insertion_point(class_scope:ostorlab.agent.message.proto.v3.use.device.LoginPassword)
  ))
_sym_db.RegisterMessage(LoginPassword)

SSHKey = _reflection.GeneratedProtocolMessageType('SSHKey', (_message.Message,), dict(
  DESCRIPTOR = _SSHKEY,
  __module__ = 'ostorlab.agent.message.proto.v3.use.device.device_pb2'
  # @@protoc_insertion_point(class_scope:ostorlab.agent.message.proto.v3.use.device.SSHKey)
  ))
_sym_db.RegisterMessage(SSHKey)

Credentials = _reflection.GeneratedProtocolMessageType('Credentials', (_message.Message,), dict(
  DESCRIPTOR = _CREDENTIALS,
  __module__ = 'ostorlab.agent.message.proto.v3.use.device.device_pb2'
  # @@protoc_insertion_point(class_scope:ostorlab.agent.message.proto.v3.use.device.Credentials)
  ))
_sym_db.RegisterMessage(Credentials)

DeviceRelay = _reflection.GeneratedProtocolMessageType('DeviceRelay', (_message.Message,), dict(
  DESCRIPTOR = _DEVICERELAY,
  __module__ = 'ostorlab.agent.message.proto.v3.use.device.device_pb2'
  # @@protoc_insertion_point(class_scope:ostorlab.agent.message.proto.v3.use.device.DeviceRelay)
  ))
_sym_db.RegisterMessage(DeviceRelay)

Duration = _reflection.GeneratedProtocolMessageType('Duration', (_message.Message,), dict(
  DESCRIPTOR = _DURATION,
  __module__ = 'ostorlab.agent.message.proto.v3.use.device.device_pb2'
  # @@protoc_insertion_point(class_scope:ostorlab.agent.message.proto.v3.use.device.Duration)
  ))
_sym_db.RegisterMessage(Duration)

Argument = _reflection.GeneratedProtocolMessageType('Argument', (_message.Message,), dict(
  DESCRIPTOR = _ARGUMENT,
  __module__ = 'ostorlab.agent.message.proto.v3.use.device.device_pb2'
  # @@protoc_insertion_point(class_scope:ostorlab.agent.message.proto.v3.use.device.Argument)
  ))
_sym_db.RegisterMessage(Argument)

Message = _reflection.GeneratedProtocolMessageType('Message', (_message.Message,), dict(
  DESCRIPTOR = _MESSAGE,
  __module__ = 'ostorlab.agent.message.proto.v3.use.device.device_pb2'
  # @@protoc_insertion_point(class_scope:ostorlab.agent.message.proto.v3.use.device.Message)
  ))
_sym_db.RegisterMessage(Message)


# @@protoc_insertion_point(module_scope)
