# pylint: disable=C0103
"""add vulnerability location

Revision ID: 746c5eb3d181
Revises:
Create Date: 2022-10-24 16:35:29.881269

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '746c5eb3d181'
down_revision = '35cd577ef0e5'
branch_labels = None
depends_on = '35cd577ef0e5'


def upgrade() -> None:
    op.add_column(
        'vulnerability',
        sa.Column('location', sa.Text)
    )


def downgrade() -> None:
    pass
